/*
 * Copyright 2012 Smallcrew, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.smallcrew.test;

import com.jayway.restassured.response.Cookie;
import com.jayway.restassured.response.Response;
import com.jayway.restassured.specification.RequestSpecification;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.webapp.WebAppContext;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;

import static com.jayway.restassured.RestAssured.expect;
import static com.jayway.restassured.http.ContentType.fromContentType;
import static fr.smallcrew.test.MavenTestUtils.resourceDirectory;
import static org.fest.assertions.Assertions.assertThat;

public abstract class BaseResourceTest {
  public static final String REST_CONTEXT = "/api";
  protected static int currentWebAppPort;
  protected static Server server;

  @BeforeClass
  public static void beforeClass() throws Exception {
    String webAppDir = MavenTestUtils.resource("webapp").getAbsolutePath();
    String extraClasspath = resourceDirectory().getAbsolutePath();
    currentWebAppPort = NetUtils.getAvailablePort(48888);
    server = new Server(currentWebAppPort);
    WebAppContext handler = new WebAppContext(webAppDir, "/");
    handler.setExtraClasspath(extraClasspath);
    server.setHandler(handler);
    server.start();
  }

  @AfterClass
  public static void afterClass() throws Exception {
    server.stop();
  }

  protected RequestSpecification requestSpec() {
    return expect().request().port(currentWebAppPort);
  }

  protected Response get(String resource, MediaType mediaType, Object... params) {
    return requestSpec().get(path(resource, mediaType), params);
  }

  protected Response delete(String resource, MediaType mediaType, Object... params) {
    return requestSpec().delete(path(resource, mediaType), params);
  }

  protected Response post(String resource, MediaType mediaType, Object body) {
    return requestSpec().contentType(fromContentType("+" + mediaType.getSubtype())).body(body).post(path(resource, mediaType));
  }

  protected RequestSpecification requestSpec(Cookie cookie) {
    return requestSpec().with().cookie(cookie);
  }

  protected Response get(TestUser user, String resource, MediaType mediaType, Object... params) {
    return requestSpec(user.sessionCookie).get(path(resource, mediaType), params);
  }

  protected Response post(TestUser user, String resource, MediaType mediaType) {
    return requestSpec(user.sessionCookie).post(path(resource, mediaType));
  }

  protected String path(String resource, MediaType mediaType) {
    StringBuilder builder = new StringBuilder();
    builder.append(REST_CONTEXT).append(resource);
    if (mediaType != null) {
      builder.append(".").append(mediaType.getSubtype());
    }
    return builder.toString();
  }

  protected void verifyException(Response response, MediaType mediaType, HttpStatus httpStatus,
                                 Class<? extends Exception> caught, String expected) {
    ResponsePath path = ResponsePath.from(response, mediaType);
    assertThat(path.getBoolean("success")).isFalse();
    assertThat(path.getInt("time")).isNotNull();
    assertThat(path.getInt("value.code")).isEqualTo(httpStatus.value());
    assertThat(path.getString("value.status")).isEqualTo(httpStatus.name());
    assertThat(path.getString("value.exception")).isEqualTo(caught.getSimpleName());
    assertThat(path.getString("value.message")).isEqualTo(expected);
    assertThat(response.getStatusCode()).isEqualTo(httpStatus.value());
    assertThat(response.getContentType()).isEqualTo(mediaType + ";charset=UTF-8");
  }

  protected TestUser createUser(String userName, String password, boolean rememberMe) {
    Response response = requestSpec().with()
      .formParameter("username", userName)
      .formParameter("password", password)
      .formParameter("rememberMe", true)
      .post(path("/login", MediaType.APPLICATION_JSON));
    ResponsePath path = ResponsePath.from(response, MediaType.APPLICATION_JSON);
    assertThat(path.getBoolean("success")).isTrue();
    assertThat(path.getString("value.name")).isEqualTo(userName);
    return new TestUser(userName, password, rememberMe, response);
  }

  protected void logout(TestUser user) {
    Response response = requestSpec().with()
      .with()
      .cookie(user.sessionCookie)
      .get(path("/logout", MediaType.APPLICATION_JSON));
    ResponsePath path = ResponsePath.from(response, MediaType.APPLICATION_JSON);
    assertThat(path.getBoolean("success")).isTrue();
  }

}
